# -*- coding=utf-8 -*-

import pyodbc

from x_py_libs.db import BaseDBHelper

class MSSQLDBHelper(BaseDBHelper):
  

    def connect(self):
        try:
            conn = pyodbc.connect(self.connect_string)
            return conn
        except Exception:
            print('mssql error')
            raise

    def fetch_returning_id(self, sql, *params):
        def callback(cur):
            return cur.fetchone()[0]

        return self.execute_sql(sql, callback, *params)

    # def fetch_rowcount(self, sql, *params):
    #     def callback(cur):
    #         return cur.rowcount

    #     return self.execute_sql(sql, callback, *params)

    # def fetch_one(self, sql, *params):
    #     def callback(cur):
    #         return cur.fetchone()

    #     return self.execute_sql(sql, callback, *params)

    # def fetch_all(self, sql, *params):
    #     def callback(cur):
    #         return cur.fetchall()

    #     return self.execute_sql(sql, callback, *params)

    def execute_sql(self, sql, callback, *params):
        conn = self.connect()

        if conn == None:
            return None

        cur = conn.cursor()

        # print('mssql:', sql, params)
        cur.execute(sql, params)

        # columns = [column[0] for column in cursor.description]
        # for row in cursor.fetchall():
        #     x = dict(zip(columns, row))
        #     results.append(x)

        rst = callback(cur)
        conn.commit()
        conn.close()
        return rst


"""
cursor = cnxn.cursor()

print ('Inserting a new row into table')
#Insert Query
tsql = "INSERT INTO Employees (Name, Location) VALUES (?,?);"
with cursor.execute(tsql,'Jake','United States'):
    print ('Successfuly Inserted!')


#Update Query
print ('Updating Location for Nikita')
tsql = "UPDATE Employees SET Location = ? WHERE Name = ?"
with cursor.execute(tsql,'Sweden','Nikita'):
    print ('Successfuly Updated!')


#Delete Query
print ('Deleting user Jared')
tsql = "DELETE FROM Employees WHERE Name = ?"
with cursor.execute(tsql,'Jared'):
    print ('Successfuly Deleted!')


#Select Query
print ('Reading data from table')
tsql = "SELECT Name, Location FROM Employees;"
with cursor.execute(tsql):
    row = cursor.fetchone()
    while row:
        print (str(row[0]) + " " + str(row[1]))
        row = cursor.fetchone()
"""



# -*- coding=utf-8 -*-

from enum import Enum
import sys
import psycopg2
from psycopg2.extras import DictCursor, RealDictCursor

from x_py_libs.db import BaseDBHelper

class PgDBHelper(BaseDBHelper):

    def connect(self):
        try:
            conn = psycopg2.connect(self.connect_string, cursor_factory=DictCursor)
            conn.set_client_encoding('UTF8')
            # print(conn)
            return conn
        except psycopg2.Error as e:
            print('psycopg2 error:', e.pgerror)
            return None

    
    def fetch_returning_id(self, sql, *params, **kw):
        sql = sql + """ RETURNING id;"""

        def callback(cur):
            rst = cur.fetchone()
            id = rst.get('id')
            return 0 if id is None else id

        return self.execute_sql(sql, callback, *params, **kw)

    def execute_many(self, sql, callback, *params, **kw):
        conn = self.connect()

        if conn == None:
            return None

        cur = conn.cursor(cursor_factory=RealDictCursor)
        # cur = conn.cursor()
        rst = None

        psycopg2.extras.execute_values(cur, sql, params, page_size=9999)

        rst = callback(cur)
        conn.commit()
        conn.close()
        # print('rst:',rst)
        return rst
        
    def execute_sql(self, sql, callback, *params, **kw):
        conn = self.connect()

        if conn == None:
            return None

        # print(sql, *params, type(params), len(params), **kw)
        is_multiple = False if kw.get('is_multiple') == None else kw['is_multiple']
        # print('--kw--:', kw, kw.get('is_multiple'), is_multiple)

        cur = conn.cursor(cursor_factory=RealDictCursor)
        # cur = conn.cursor()
        rst = None

        if is_multiple:
            psycopg2.extras.execute_values(cur, sql, params, page_size=9999)
        else:
            if params is not None:
                # print('sql:', cur.mogrify(sql, *params))
                cur.execute(sql, *params)
            else:
                cur.execute(sql)

        # rst = cur.fetchone()

        rst = callback(cur)
        conn.commit()
        conn.close()
        # print('rst:',rst)
        return rst

    def callproc(self, proc_name, *params):
        conn = self.connect()

        if conn == None:
            return None

        cur = conn.cursor()
        cur.callproc(proc_name, *params)
        rst = cur.fetchall()

        conn.commit()
        conn.close()
        return rst


"""
    # def execute_values(self, sql, *params):
    #     conn = db_helper.connect()

    #     if conn == None:
    #         return None

    #     print(params, type(params), list(params))

    #     cur = conn.cursor()
    #     psycopg2.extras.execute_values(cur, sql, params)
    #     rst = cur.rowcount
    #     conn.commit()
    #     conn.close()
    #     return rst
"""



# -*- coding=utf-8 -*-

from .redis_helper import RedisHelper

class AppRedisHelper(object):

    redis_helper = None

    key_prefix = ''

    def __init__(self, *args, **kwargs):
        try:
            host = kwargs.get('host')
            password = kwargs.get('password')
            port = kwargs.get('port')

            self.key_prefix = kwargs.get('key_prefix')

            self.redis_helper = RedisHelper(host=host, port=port, password=password)
            
        except Exception:
            pass

        return super().__init__(*args, **kwargs)

    def get_key_prefix(self, key):
        return self.key_prefix + key

    def set(self, key, value):
        try:
            self.redis_helper.set(self.get_key_prefix(key), value)
        except Exception:
            pass

    def get(self, key):
        try:
            return self.redis_helper.get(self.get_key_prefix(key))
        except Exception:
            return None

    def delete(self, key):
        try:
            return self.redis_helper.delete(self.get_key_prefix(key))
        except Exception:
            return None

    def hash_mset(self, key, value):
        try:
            self.redis_helper.hash_mset(self.get_key_prefix(key), value)
        except Exception:
            pass

    def hash_get_all(self, key):
        try:
            return self.redis_helper.hash_get_all(self.get_key_prefix(key))
        except Exception:
            return None



# -*- coding=utf-8 -*-

# https://cloud.tencent.com/developer/article/1067121

from captcha.image import ImageCaptcha, random_color
from random import randint
import base64
from io import BytesIO


CHAR_LIST = [
        '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
        #'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 
        #'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
        #'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 
        #'J', 'K', 'L', 'M', 'N', 'O', 
        #'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'
        ]

class CaptchaHelper(object):

    @staticmethod
    def generate(length=4, width=160, height=60, fonts=None, font_sizes=None, char_list=CHAR_LIST, create_noise_dots=True, create_noise_curve=True):
        
        chars = ''.join([(char_list[randint(0, len(char_list) - 1)])for i in range(length)])
        
        # image = ImageCaptcha(width=width, height=height, fonts=fonts, font_sizes=font_sizes).generate_image(chars)

        # generate_image的实现方法，可以通过重写此方法来实现自定义验证码样式。
        # create_captcha_image(chars, color, background) 
 
        # 生成验证码干扰点。
        # create_noise_dots(image, color, width=3, number=30) 
        
        # 生成验证码干扰曲线。
        # create_noise_curve(image, color) 

        background = random_color(238, 255)
        color = random_color(10, 200, randint(220, 255))

        ic = ImageCaptcha(width=width, height=height, fonts=fonts, font_sizes=font_sizes)
        image = ic.create_captcha_image(chars, color, background) 

        if create_noise_dots:
            ic.create_noise_dots(image, color, width=3, number=30) 
        if create_noise_dots:
            ic.create_noise_curve(image, color) 

        buffered = BytesIO()
        image.save(buffered, format="JPEG")
        img_str = base64.b64encode(buffered.getvalue())
        # print(chars ,type(image),img_str)
        return chars, img_str
        # image.show()

# CaptchaHelper.generate()

# s, v = gvcode.generate() #序列解包
# s.show() #显示生成的验证码图片
# print(v) #打印验证码字符串
 
# image.show()
 
        # self.write(simplejson.dumps({'code': 0, 'img': stream.getvalue().encode('base64')}))
    #这里是将stream的值进行了一次base64的编码



# -*- coding=utf-8 -*-

import tornado.web

from x_py_libs.helpers import UtilitiesHelper, CryptoHelper


class BaseTornadoHandler(tornado.web.RequestHandler):
  
    app_config = None

    user_id = None
    ignore_authorization = False
    ignore_sign_in_authorization = False

    authorization_config = None
    authorization_token_key = 'x-api-token'

    env = 'dev'

    cache_helper = None

    decryptSC = True

    page_index_param_name = 'pageIndex'
    page_size_param_name = 'pageSize'

    def __init__(self, *args, **kwargs):
        # print('__init__ super')
        super().__init__(*args, **kwargs)

    # def initialize(self, *args, **kwargs):
    #     print('initialize super')
    #     super().initialize(**kwargs)

    def __verify_token(self):
        authorization_token = self.request.headers.get(self.authorization_token_key)

        try:
            if authorization_token is None:
                return False

            t = authorization_token.split('|')
            token = CryptoHelper.decrypt_by_aes(t[0]).split('_')
            decrypt_ts = CryptoHelper.decrypt_by_aes(token[2])
            ts = token[0] 
            md5_ts = CryptoHelper.encrypt_by_md5(ts).upper()

            if ts != decrypt_ts or md5_ts != t[1]:
                return False

            self.user_id = int(token[1])

            if not self.ignore_sign_in_authorization and self.user_id == 0:
                return False

            return True

        except Exception as e:
            print(e)
            return False

    def set_default_headers(self):
        if self.env == 'dev':
            self.set_header('Access-Control-Allow-Credentials', 'False')
            origin = self.request.headers.get('Origin', '')
            # 设置允许的'origin'，只设置'*'时某些特定情况下会失败故最好优先获取请求的域加入允许组中
            self.set_header('Access-Control-Allow-Origin', origin or '*')
            self.set_header('Access-Control-Allow-Headers', '*')
            self.set_header('Access-Control-Allow-Methods', 'POST, GET, PUT, DELETE, OPTIONS')
        self.set_header('Content-Type', '*')
        pass

    def prepare(self, *args, **kwargs):
        super().prepare()
        if self.request.method != 'OPTIONS':
            if not self.ignore_authorization:
                if not self.__verify_token():
                    self.write_response_base(code=-999999, msg='Access Deined!')


    def head(self, *args, **kwargs):
        pass

    def get(self, *args, **kwargs):
        pass

    def post(self, *args, **kwargs):
        pass

    def delete(self, *args, **kwargs):
        pass

    def patch(self, *args, **kwargs):
        pass

    def put(self, *args, **kwargs):
        pass

    def options(self, *args, **kwargs):
        self.set_status(204)
        self.finish()

    def write_decrypt_response(self, code=1, msg='ok', data=None, json=1, ext=None):
        self.write_response_base(code=code,msg=msg,data=data, encrypt=False, json=json,ext=ext)

    def write_response_base(self, code=1, msg='ok', data=None, encrypt=True, json=1, ext=None):
        if data is not None:
            # data = data if type(data) is str else UtilitiesHelper.json_dumps_dict(data)
            data = CryptoHelper.encrypt_by_aes(data) if encrypt else data
        else:
            data = ''

        resp = {
            'code': code,
            'msg': msg,
            'data': data,
            'j': json
        }

        if ext is not None:
            resp[ext.get('key')] = ext.get('value')

        self.write(UtilitiesHelper.json_dumps_dict(resp))
        self.finish()

    # def build_response(self, keys=[], data=None, encrypt=True):
    #     response = ApiConf.get_response(keys)
    #     self.write_response_base(response['code'], response['msg'], data, encrypt=encrypt)

    # def write_auth_invalid_response(self):
    #     self.build_response(['common', 'auth-invalid'])

    def get_request_query_argument(self, arg_name=None):
        if len(self.request.arguments) == 0:
            return None

        if arg_name is None:
            try:
                return self.get_query_argument('data')
            except Exception as e:
                return UtilitiesHelper.json_dumps({ k: self.get_argument(k) for k in self.request.arguments })
                # return self.request.arguments

        return self.get_query_argument(arg_name)

    def get_decrypt_request_query_argument(self, arg_name=None, decrypt=None):
        sc = self.get_request_query_argument(arg_name=arg_name)
        if sc is not None:
            decrypt = self.decryptSC if decrypt is None else decrypt
            if decrypt and type(sc) is str:
                sc = CryptoHelper.decrypt_by_aes(sc)
            if sc is not None and type(sc) is str:
                return UtilitiesHelper.json_loads(sc)
            else:
                return sc
        return None

    def get_decrypt_request_body_data(self, key=None, decrypt=False):
        try:
            data = UtilitiesHelper.json_loads(self.request.body)
            if key is not None and key != '':
                data = data.get(key)
            if decrypt:
                data = CryptoHelper.decrypt_by_aes(data)
                data = UtilitiesHelper.json_loads(data)
            return data
        except Exception as e:
            print('get decrypt request body data error:', e)
            return None

    def write_sc_list_response(self, query, arg_name=None, decrypt=None, rst=None, cnt=0):
        data = None
        if rst is None:
            data = self.get_decrypt_request_query_argument(arg_name=arg_name, decrypt=decrypt)
            if data is not None:
                page_index = data.get(self.page_index_param_name)
                page_size = data.get(self.page_size_param_name)
                if page_index is not None and page_size is not None:
                    data['page_index'] = page_index
                    data['page_size'] = page_size
            rst, cnt = query(data)
        data = {
            'list': rst,
            'total': cnt
        }

        self.write_response_base(data=data, encrypt=False)

    def write_cache_response(self, cache_key, query, simple=True, keys=[]):
        cache_data = None

        if self.cache_helper is not None:
            cache_data = self.cache_helper.get(cache_key)

        # cache_data = None
        if cache_data is None:

            cache_data = query()

            if simple:
                # data = query()
                if type(cache_data) is tuple:
                    cache_data = dict(map(lambda _key, _data: (_key, _data if _data is not None else 'undefined'), keys, cache_data))

                cache_data = {
                    cache_key: cache_data
                }

            cache_data = UtilitiesHelper.json_dumps(cache_data)
            self.cache_helper.set(cache_key, cache_data)

        self.write_response_base(data=cache_data, encrypt=False)
        super().get()



from .utilities_helper import *

import sys

class ModelTransferHelper(object):
  
    _globals = None

    def __init__(self, class_name):
        self.obj = self._globals[class_name]()
        self.keys_relation = []
        # self.obj = []

    def transfer_to_py(self, data):
        # model = dict(map(lambda x: (x[1], '' if data.get(x[0]) is None else data[x[0]]), self.keys_relation))
        for key in self.keys_relation:
            ts_attr = key[0]
            d = data.get(ts_attr)
            setattr(self.obj, key[1], d)

        return self.obj

    def auto_transfer(self, data):
        keys = list(data.keys())
        for key in keys:
            snake_key = UtilitiesHelper.camel_2_snake(key)
            # print(key, snake_key)
            setattr(self.obj, snake_key, data.get(key))
        return self.obj


# -*- coding=utf-8 -*-


class ICacheHelper(object):

    def set(self, key, value):
        pass

    def get(self, key):
        pass

    def delete(self, key):
        pass

    def hash_mset(self, key, value):
        pass

    def hash_get_all(self, key):
        pass



# -*- coding=utf-8 -*-

import requests
import shutil
import os


# PROXY = '127.0.0.1:1081'
# HTTP_PROXY_PORT = ':1080'
HTTP_PROXIES = {
    'http': 'http://127.0.0.1:1081',
    'https': 'http://127.0.0.1:1081'
}
SOCKS_PROXIES = {
    'http': 'socks5://127.0.0.1:1080',
    'https': 'socks5://127.0.0.1:1080'
}

HEADERS = {
    'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3394.0 Safari/537.36',
    # 'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8'
}


class RequestsHelper(object):

    @staticmethod
    def get(url, params=None):
        return requests.get(url, **({} if params is None else params))

    @staticmethod
    def post(url, **kwargs):
        return requests.post(url, **kwargs)

    @staticmethod
    def fetch_url(url, params=None):
        return requests.get(url, **({} if params is None else params)).content

    @staticmethod
    def fetch_stream(url, return_raw=True):
        response = requests.get(url, stream=True)
        # return requests.get(url, stream=True).content
        if return_raw:
            response.raw.decode_content = True
            return response.raw

        return response

    @staticmethod
    def fetch_page(url, refer_url=None, http_request=None, use_proxy=False, socks_proxy=False, decode_content=False, cookie=None):
        # proxy = '127.0.0.1:1080'
        # proxies = {
        #     'http': 'http://' + proxy,
        #     'https': 'https://' + proxy
        # }

        _headers = HEADERS

        if refer_url is not None:
            _headers['Referer'] = refer_url

        if http_request is not None:
            _headers['x-requested-with'] = http_request

        if cookie is not None:
            _headers['Cookie'] = cookie

        # params = {
        #     'headers': HEADERS
        # }

        # __proxies = HTTP_PROXIES if use_proxy else {}
        _proxies = None
        if use_proxy:
            if not socks_proxy:
                _proxies = HTTP_PROXIES
            else:
                _proxies = SOCKS_PROXIES

        try:
            # r = requests.get(url, **params)
            r = requests.get(url, proxies=_proxies, headers=_headers)
            # r.encoding = 'utf-8'

            if decode_content:
                return r.content.decode('utf-8')

            return r

            # print(r.content.decode('utf-8'))
        except Exception as e:
            print(e)
            return None

    @staticmethod
    def save_fetch(url, save_path):
        rsp = RequestsHelper.fetch_stream(url, return_raw=False)
        # with open(save_path, 'x') as f:
        #     shutil.copyfileobj(raw, f)

        if rsp.status_code == 200:
            with open(save_path, 'wb') as f:
                for chunk in rsp.iter_content(1024):
                    f.write(chunk)

    @staticmethod
    def download(url, save_path, proxies=None, timeout=30, override=False, log_size_lmt=1024, use_proxy=False, socks_proxy=False):

        file_name = url.split("/")[-1].split("?")[0]

        if not os.path.exists(save_path):
            os.makedirs(save_path)

        file_path = os.path.join(save_path, file_name)
        print("Downloading %s from %s.\n" % (file_path, url))

        if os.path.exists(file_path) and not override:
            return

        # if use_proxy:
        #     proxies = HTTP_PROXIES
        # __proxies = None
        if use_proxy:
            if not socks_proxy:
                proxies = HTTP_PROXIES
            else:
                proxies = SOCKS_PROXIES

        try:
            resp = requests.get(url, stream=True, proxies=proxies, timeout=timeout, headers=HEADERS)
            # print(resp.headers)
            if resp.status_code == 404:
                print("File Not Found when retrieve %s.\n" % url)
                raise Exception("404")
            if resp.status_code == 403:
                print("Access Denied when retrieve %s.\n" % url)
                raise Exception("Access Denied")
            # else:
            #     print(resp.status_code)

            # content_length = resp.headers['Content-Length']
            content_length = resp.headers.get('Content-Length')
            # print(type(content_length))
            if content_length is not None:
                # print('content_length:', content_length)
                if int(content_length) <= log_size_lmt:
                    with open(os.path.join(save_path, 's.txt'), 'w+', encoding='utf-8') as f:
                        f.write(url + ' ---------- ' + file_path + ' --------- ' + str(vars(resp)))

            with open(file_path, 'wb') as fh:
                for chunk in resp.iter_content(chunk_size=1024):
                    fh.write(chunk)
        except Exception as err:
            raise err
            print(err)
        # finally:
        #     return file_path



# -*- coding=utf-8 -*-

# __all__ = [
#     'db_helper_mssql',
#     'db_helper_pg'
# ]

from .db_helper_base import *
from .db_helper_mssql import *
from .db_helper_pg import *
from .db_helper_mysql import *



# -*- coding=utf-8 -*-

from distutils.core import setup

from setuptools import setup, find_packages

setup(
    name="x_py_libs",
    version="0.5.20",
    keywords=["pip", "self", "python", "libs"],
    description="self python libs",
    long_description="self python libs",
    license="MIT Licence",

    url="https://github.com/xiaxiazhu119",
    author="xiaxiazhu",
    author_email="xiaxiazhu147@gmail.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=[
        'psycopg2', 'pyodbc', 'sqlalchemy',
        'tornado',
        'Flask', 'Flask_Cors', 'Flask_RESTful',
        'redis',
        'six', 'pymediainfo', 'itsdangerous', 'Werkzeug',
        'requests',
        # 'pycrypto',
        'pycryptodome',
        'captcha','graphic-verification-code',
        'mysql_connector_repackaged',
        'pymysql'
    ]
)



from mysql.connector import connect, Error
import pymysql

from enum import Enum
 
from x_py_libs.db import BaseDBHelper

class MySqlConnectTypeEnum(Enum):
    PyMySql = 1
    MySqlConnector = 2

class MySqlDBHelper(BaseDBHelper):

    def connect(self):
        try:
            user = self.params.get('user')
            password = self.params.get('password')
            host = self.params.get('host')
            port = self.params.get('port')
            database = self.params.get('database')

            connect_method = self.params.get('connect_method')

            conn = None
            connect_method = MySqlConnectTypeEnum.PyMySql if connect_method is None else connect_method

            connect_params = {
                'user':user,
                'password': password,
                'host':host,
                'port': int(port),
                'database':database
            }
            connect_func = None

            if connect_method == MySqlConnectTypeEnum.PyMySql:
                connect_func = pymysql.connect;
                connect_params['cursorclass'] = pymysql.cursors.DictCursor
                # conn = pymysql.connect(host, user, password, database, port)
            elif connect_method == MySqlConnectTypeEnum.MySqlConnector:
                connect_func = connect.connect;
                connect_params['dictionary'] = True
                connect_params['use_pure'] = False
                # conn = connect.connect(user=user, password=password,
                #                 host=host,
                #                 port=port,
                #                 database=database,
                #                 use_pure=False)
            conn = connect_func(**connect_params)
            return conn
        except Error as e:
            print(e)
            return None

    def fetch_returning_id(self, sql, *params, **kw):

        def callback(cur):
            return cur.lastrowid

        return self.execute_sql(sql, callback, *params, **kw)

    # def fetch_rowcount(self, sql, *params, **kw):
    #     return super().fetch_rowcount(sql, *params, **kw)

    def execute_many(self, sql, callback, params, **kw):
        conn = self.connect()
        cur = conn.cursor()
        cur.executemany(sql, params)
        rst = callback(cur)
        conn.commit()
        conn.close()
        # print('rst:',rst)
        return rst

    def execute_sql(self, sql, callback, *params, **kw):
        conn = self.connect()

        if conn == None:
            return None

        # cur = conn.cursor(cursor_factory=RealDictCursor)
        cur = conn.cursor()
        rst = None

        # print('sql:', cur.mogrify(sql, *params))

        cur.execute(sql, *params)

        rst = callback(cur)
        conn.commit()
        conn.close()
        # print('rst:',rst)
        return rst

    def callproc(self, proc_name, *params):
        conn = self.connect()

        if conn == None:
            return None

        cur = conn.cursor()
        cur.callproc(proc_name, *params)
        rst = cur.fetchall()

        conn.commit()
        conn.close()
        return rst
